/*
 * misc.hpp - Kukatz 3D
 * Copyright (c) 2010, 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#ifndef MISC_HPP_INCLUDED
#define MISC_HPP_INCLUDED

#include <vector>
#include <cmath>
#include <cstdlib>

#include <SFML/System/Vector2.hpp>
#include <SFML/System/Vector3.hpp>
#include <SFML/Graphics/Color.hpp>

#include "opengl.hpp"

inline sf::Vector3f itof3(const sf::Vector3i& i)
{
	return sf::Vector3f(i.x, i.y, i.z);
}

inline sf::Vector3i ftoi3(const sf::Vector3f& f)
{
	return sf::Vector3i(f.x + 0.5f, f.y + 0.5f, f.z + 0.5f); // for correct rounding
}

inline sf::Vector2i ftoi2(const sf::Vector2f& f)
{
	return sf::Vector2i(f.x + 0.5f, f.y + 0.5f); // for correct rounding
}

inline sf::Vector3i crossprod(const sf::Vector3i& v1, const sf::Vector3i& v2)
{	// xyzzy
	return sf::Vector3i(
		v1.y * v2.z - v1.z * v2.y,
		v1.z * v2.x - v1.x * v2.z,
		v1.x * v2.y - v1.y * v2.x
	);
}

inline sf::Vector3f crossprod(const sf::Vector3f& v1, const sf::Vector3f& v2)
{	// xyzzy
	return sf::Vector3f(
		v1.y * v2.z - v1.z * v2.y,
		v1.z * v2.x - v1.x * v2.z,
		v1.x * v2.y - v1.y * v2.x
	);
}

inline sf::Vector3f crossprod(
	float x1, float y1, float z1,
	float x2, float y2, float z2)
{	// xyzzy
	return sf::Vector3f(
		y1 * z2 - z1 * y2,
		z1 * x2 - x1 * z2,
		x1 * y2 - y1 * x2
	);
}

inline sf::Vector3f transform(const sf::Vector3f& v, float m[16])
{
	return sf::Vector3f(
			v.x * m[0] + v.y * m[4] + v.z * m[8] + m[12],
			v.x * m[1] + v.y * m[5] + v.z * m[9] + m[13],
			v.x * m[2] + v.y * m[6] + v.z * m[10] + m[14]
		);
}

inline void get_AABB(const std::vector< sf::Vector3f >& v,
	sf::Vector3f& n, sf::Vector3f& p)
{
	if (v.empty())
	{
		n.x = n.y = n.z = p.x = p.y = p.z = 0;
	}
	else
	{
		n = v[0];
		p = v[0];
		
		for (size_t i = 1; i < v.size(); ++i)
		{
			if (v[i].x < n.x)
			{
				n.x = v[i].x;
			}
			
			if (v[i].y < n.y)
			{
				n.y = v[i].y;
			}
			
			if (v[i].z < n.z)
			{
				n.z = v[i].z;
			}
			
			if (v[i].x > p.x)
			{
				p.x = v[i].x;
			}
			
			if (v[i].y > p.y)
			{
				p.y = v[i].y;
			}
			
			if (v[i].z > p.z)
			{
				p.z = v[i].z;
			}
		}
	}
}

inline std::vector< sf::Vector3f > get_corners(const sf::Vector3f& n,
	const sf::Vector3f& p)
{
	std::vector< sf::Vector3f > ret;
	ret.resize(8);
	
	
	ret[0] = n;
	
	
	ret[1] = n;
	ret[1].x = p.x;
	
	ret[2] = n;
	ret[2].y = p.y;
	
	ret[3] = n;
	ret[3].z = p.z;
	
	
	ret[4] = p;
	ret[4].x = n.x;
	
	ret[5] = p;
	ret[5].y = n.y;
	
	ret[6] = p;
	ret[6].z = n.z;
	
	
	ret[7] = p;
	
	return ret;
}

inline float length(const sf::Vector3f& v)
{
	return sqrtf(v.x * v.x + v.y * v.y + v.z * v.z);
}

inline void clamp_to_distance(const sf::Vector3f& o, float d, sf::Vector3f& v)
{
	v -= o;
	v *= (d / length(v));
	v += o;
}

inline unsigned int step_dist(const sf::Vector3i& v1, const sf::Vector3i& v2)
{
	return abs(v1.x - v2.x) + abs(v1.y - v2.y) + abs(v1.z - v2.z);
}

inline sf::Vector3f lerp(const sf::Vector3f& v1, const sf::Vector3f& v2,
	float ratio)
{
	if (ratio == 0.0) return v1;
	if (ratio == 1.0) return v2;
	
	return v1 + (v2 - v1) * ratio; // faster than v1*(1.0-ratio) + v2*ratio
}

inline sf::Vector3f normalize(const sf::Vector3f& v)
{
	return v / sqrtf(v.x * v.x + v.y * v.y + v.z * v.z);
}

inline void reset_material()
{
	float arr[] = {1.0f, 1.0f, 1.0f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, arr);
	glMaterialfv(GL_FRONT, GL_SPECULAR, arr);
}

inline void set_material(const sf::Color& col)
{
	float arr[] = {(float)col.r/255.f,(float)col.g/255.f,(float)col.b/255.f,(float)col.a/255.f};
	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,arr);
	glMaterialfv(GL_FRONT,GL_SPECULAR,arr);
}

#endif // MISC_HPP_INCLUDED
