/*
 * font.cpp - Kukatz 3D
 * Copyright (c) 2011 - TÖRÖK Attila (torokati44@gmail.com)
 *
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 *	 1. The origin of this software must not be misrepresented; you must not
 *			claim that you wrote the original software. If you use this software
 *			in a product, an acknowledgment in the product documentation would be
 *			appreciated but is not required.
 *
 *	 2. Altered source versions must be plainly marked as such, and must not be
 *			misrepresented as being the original software.
 *
 *	 3. This notice may not be removed or altered from any source
 *			distribution.
 */

#include "font.hpp"

#include <cstring>
#include <iostream>

#include "resources.hpp"

Font::Font(std::string texture, const unsigned char* data, sf::Uint32 data_size):
	texture_ptr(Resources::instance().textures[texture])
{
	unsigned char* current = (unsigned char*)data;
	unsigned char* end = current;
	end += data_size; // AFTER the last byte
	
	assert(data_size >= 9);
	assert(!strncmp((char*)current, "BMF\003", 4));
	current += 4;
	
	while (current < end)
	{
		sf::Uint8 block_type = *((sf::Uint8*)current);
		current += 1;
		sf::Uint32 block_size = *((sf::Uint32*)current);
		current += 4;
		unsigned char* block_end = current;
		block_end += block_size;
		
		switch (block_type)
		{
			case 1: // info
			{
				size = *((sf::Uint16*)current); // font size
				current += block_size;
			}
			break;
			case 2: // common
			{
				line_height = *((sf::Uint16*)current);
				current += 2;
				
				base = *((sf::Uint16*)current);
				current += 2;
				
				current += 11; // channels, size, etc... we don't need it
			}
			break;
			case 3: // pages
			{
				current += block_size; // we don't need anything from this block...
			}
			break;
			case 4: // chars
			{
				while (current < block_end)
				{
					sf::Uint32 id;
					
					id = *((sf::Uint32*)current);
					current += 4;
					
					glyph_t glyph;
					
					glyph.x = *((sf::Uint16*)current);
					current += 2;
					
					glyph.y = *((sf::Uint16*)current);
					current += 2;
					
					glyph.w = *((sf::Uint16*)current);
					current += 2;
					
					glyph.h = *((sf::Uint16*)current);
					current += 2;
					
					glyph.xo = *((sf::Int16*)current);
					current += 2;
					
					glyph.yo = *((sf::Int16*)current);
					current += 2;
					
					glyph.xa = *((sf::Int16*)current);
					current += 2;
					
					current += 2; // page, channel, not interesting...
					
					glyphs.insert(std::make_pair< sf::Uint32, glyph_t >(id, glyph));
				}
			}
			break;
			case 5: // kerning pairs
			{
				while (current < block_end)
				{
					sf::Uint32 first = *((sf::Uint32*)current);
					current += 4;
					
					sf::Uint32 second = *((sf::Uint32*)current);
					current += 4;
					
					sf::Int16 amount = *((sf::Int16*)current);
					current += 2;
					
					glyphs[first].kernings.push_back(std::make_pair< sf::Uint32, sf::Int16 >(second, amount));
				}
			}
			break;
			default:
			{
				std::cerr << "Unexpected block type (" << (sf::Uint16)block_type << ") when loading font!\n";
			}
			break;
		}
		
		if (current != block_end)
		{
			std::cerr << "Block size mismatch during loading font: !\n";
		}
	}
	
	if (current != end)
	{
		std::cerr << "Data size mismatch during loading font!\n";
	}
}

Font::~Font()
{
	
}
